Here are the available documentation sources I can search:

{% for doc in docs %}
<docs-{{ loop.index }}>
  <docs-name>{{ doc.name }}</docs-name>
  <docs-summary>{{ doc.summary }}</docs-summary>
</docs-{{ loop.index }}>
{% endfor %}
